-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Race AI Aggression Modifier'
C.description = 'Changes the AI properties during the race, at pathnodes.'
C.category = 'logic'
C.tmpSecondPassFlag = true
C.color = im.ImVec4(1, 1, 0, 0.75)
C.pinSchema = {
  {dir = 'in', type = 'flow', name = 'flow'},
  {dir = 'in', type = 'table', name = 'raceData', tableType = 'raceData', description = 'Data from the race for other nodes to process.'},
  {dir = 'in', type = 'number', name = 'vehId', description = 'The vehicle that should be tracked.'},
  {dir = 'in', type = 'number', name = 'oldAggression', description = 'The default AI aggression value to use.'},
  {dir = 'in', type = 'number', name = 'oldTurnForceCoef', description = 'The default AI turn force coef value to use.'},

  {dir = 'out', type = 'flow', name = 'flow'},
  {dir = 'out', type = 'string', name = 'nodeName', description = 'Name of the nearest pathnode.'}
}

C.tags = {'scenario'}

function C:init(mgr, ...)
  self.inside = false
  self.lastId = nil
  self.data.debugMode = false
end

function C:work(args)
  self.race = self.pinIn.raceData.value
  self.data.debugMode = true
  if not self.race or not self.pinIn.vehId.value then return end

  local obj = be:getObjectByID(self.pinIn.vehId.value)
  if not obj then return end

  local vehPos = obj:getPosition()
  local bestId
  local bestDist = math.huge

  for id, pn in pairs(self.race.path.pathnodes.objects) do
    local dist = vehPos:squaredDistance(pn.pos)
    if dist < bestDist then
      bestDist = dist
      bestId = id
    end
  end

  if bestId then
    if self.lastId ~= bestId then
      self.lastId = bestId
      self.inside = false
    end

    local pn = self.race.path.pathnodes.objects[bestId]
    local radius = pn.customFields:get('effectRadius') or pn.radius

    if not self.inside and vehPos:squaredDistance(pn.pos) <= square(radius) then -- entered pathnode radius
      self.inside = true
      if pn.customFields:has('aggression') then
        local aggression = pn.customFields:get('aggression')
        if aggression == -1 then aggression = self.pinIn.oldAggression.value or 1 end

        obj:queueLuaCommand('ai.setAggression('..aggression..')')
        if self.data.debugMode then print('vehId ['..self.pinIn.vehId.value..'] aggression: '..aggression) end
      end
      if pn.customFields:has('turnForceCoef') then
        local turnForceCoef = pn.customFields:get('turnForceCoef')
        if turnForceCoef == -1 then turnForceCoef = self.pinIn.oldTurnForceCoef.value or 2 end

        obj:queueLuaCommand('ai.setParameters({turnForceCoef = '..turnForceCoef..'})')
        if self.data.debugMode then print('vehId ['..self.pinIn.vehId.value..'] turn force coef: '..turnForceCoef) end
      end
    elseif self.inside and vehPos:squaredDistance(pn.pos) > square(radius) then -- exited pathnode radius
      self.inside = false
      if pn.customFields:get('resetParameters') == 'true' then
        local oldAggression = self.pinIn.oldAggression.value or 1
        local oldTurnForceCoef = self.pinIn.oldTurnForceCoef.value or 2

        obj:queueLuaCommand('ai.setAggression('..oldAggression..')')
        if self.data.debugMode then print('vehId ['..self.pinIn.vehId.value..'] old aggression: '..oldAggression) end

        obj:queueLuaCommand('ai.setParameters({turnForceCoef = '..oldTurnForceCoef..'})')
        if self.data.debugMode then print('vehId ['..self.pinIn.vehId.value..'] old turn force coef: '..oldTurnForceCoef) end
      end
    end

    self.pinOut.nodeName.value = pn.name
  end

  self.pinOut.flow.value = self.pinIn.flow.value
end

return _flowgraph_createNode(C)