-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Get Complete Path'
C.description = 'Returns the path table containing every node from start to finish.'
C.color = im.ImVec4(1, 1, 0, 0.75)
C.category = 'once_instant'

C.pinSchema = {
  { dir = 'in', type = 'flow', name = 'flow' },
  { dir = 'in', type = 'flow', name = 'reset', impulse = true },
  { dir = 'in', type = 'table', name = 'raceData', description = 'Race path data.' },
  { dir = 'in', type = 'string', name = 'startNode', hidden = true, description = 'Map node to start the path from.' },
  { dir = 'in', type = 'string', name = 'endNode', hidden = true, description = 'Map node to end the path at.' },

  { dir = 'out', type = 'flow', name = 'flow' },
  { dir = 'out', type = 'table', name = 'navPathData', description = 'Complete map path table.' }
}

function C:init()
  self:reset()
end

function C:reset()
  self.pinOut.flow.value = false
end

function C:_executionStopped()
  self:reset()
end

function C:work()
  if self.pinIn.reset.value then
    self:reset()
  end
  if self.pinIn.flow.value and not self.pinOut.flow.value then
    local navPathData = {}
    if self.pinIn.raceData.value then
      local aiPath = deepcopy(self.pinIn.raceData.value.aiPath)
      local firstNode = map.findClosestRoad(vec3(self.pinIn.raceData.value.path.pathnodes.sorted[1].pos))
      table.insert(aiPath, 1, firstNode)
      table.insert(navPathData, firstNode)

      for i = 1, #aiPath - 1 do
        local section = map.getPath(aiPath[i], aiPath[i + 1])
        local flag = true
        for _, v in ipairs(section) do
          if not flag then
            table.insert(navPathData, v)
          end
          flag = false
        end
      end
    elseif self.pinIn.startNode.value and self.pinIn.endNode.value then
      navPathData = map.getPath(self.pinIn.startNode.value, self.pinIn.endNode.value)
    end

    self.pinOut.navPathData.value = navPathData
    self.pinOut.flow.value = true
  end
end

return _flowgraph_createNode(C)